package org.apache.maven.plugin.linkcheck.validation;

/* ====================================================================
 *   Copyright 2001-2005 The Apache Software Foundation.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author <a href="mailto:bwalding@apache.org">Ben Walding</a>
 * @author <a href="mailto:carlos@apache.org">Carlos Sanchez</a>
 * @author <a href="mailto:aheritier@apache.org">Arnaud Heritier</a>
 * @version $Id: LinkValidatorManager.java 368935 2006-01-14 00:28:50Z aheritier $
 */

public class LinkValidatorManager
    implements Serializable
{
    /**
     * Log for debug output
     */
    private static Log LOG = LogFactory.getLog( LinkValidatorManager.class );

    private List validators = new LinkedList();

    private String[] excludes = new String[0];

    private Map cache = new HashMap();

    public LinkValidatorManager()
    {
    }

    public List getValidators()
    {
        return validators;
    }

    /**
     * Returns the exclude.
     * @return String
     * @deprecated use getExcludes()
     */
    public String getExclude()
    {
        return excludes[0];
    }

    /**
     * Sets the exclude.
     * @param exclude The exclude to set
     * @deprecated use setExcludes()
     */
    public void setExclude( String exclude )
    {
        this.excludes = new String[] { exclude };
    }

    /**
     * Returns the excludes.
     * @return String[]
     */
    public String[] getExcludes()
    {
        return excludes;
    }

    /**
     * Sets the excludes.
     * @param excludes The excludes to set
     */
    public void setExcludes( String[] excludes )
    {
        this.excludes = excludes;
    }

    public void addLinkValidator( LinkValidator lv )
    {
        validators.add( lv );
    }

    public LinkValidationResult validateLink( LinkValidationItem lvi )
        throws Exception
    {
        {
            LinkValidationResult status = getCachedResult( lvi );
            if ( status != null )
            {
                return status;
            }
        }

        for ( int i = 0; i < excludes.length; i++ )
        {
            if ( excludes[i] != null && lvi.getLink().startsWith( excludes[i] ) )
            {
                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( "Excluded " + lvi.getLink() );
                }
                return new LinkValidationResult( LinkValidationResult.VALID, false, "" );
            }
        }
        Iterator iter = validators.iterator();
        LinkValidator lv;
        Object resourceKey;
        LinkValidationResult lvr;
        while ( iter.hasNext() )
        {
            lv = (LinkValidator) iter.next();
            resourceKey = lv.getResourceKey( lvi );
            if ( resourceKey != null )
            {
                if ( LOG.isDebugEnabled() )
                    LOG.debug( lv.getClass().getName() + " - Checking link " + lvi.getLink() );
                lvr = lv.validateLink( lvi );
                if ( lvr.getStatus() == LinkValidationResult.NOTMINE )
                {
                    continue;
                }
                setCachedResult( resourceKey, lvr );
                return lvr;
            }
        }
        lv = null;
        resourceKey = null;
        lvr = null;
        LOG.error( "Unable to validate link : " + lvi.getLink() );
        return new LinkValidationResult( LinkValidationResult.UNKNOWN, false, "No validator found for this link" );
    }

    public void loadCache( String cacheFilename )
    {
        try
        {
            File f = new File( cacheFilename );
            if ( f.exists() )
            {
                ObjectInputStream is = new ObjectInputStream( new FileInputStream( cacheFilename ) );
                this.cache = (Map) is.readObject();
                is.close();
            }
        }
        catch ( InvalidClassException e )
        {
            LOG.warn( "Your cache is incompatible with this new release of the plugin. It will be recreated." );
        }
        catch ( Throwable t )
        {
            LOG.error( "Unable to load the cache", t );
        }
    }

    public void saveCache( String cacheFilename )
    {
        try
        {
            // Remove non-persistent items from cache
            Map persistentCache = new HashMap();
            Iterator iter = this.cache.keySet().iterator();
            Object resourceKey;
            while ( iter.hasNext() )
            {
                resourceKey = iter.next();
                if ( ( (LinkValidationResult) cache.get( resourceKey ) ).isPersistent() )
                {
                    persistentCache.put( resourceKey, cache.get( resourceKey ) );
                    if ( LOG.isDebugEnabled() )
                        LOG.debug( "[" + resourceKey + "] with result [" + cache.get( resourceKey )
                            + "] is stored in the cache." );
                }
            }
            File cacheFile = new File( cacheFilename );
            File dir = cacheFile.getParentFile();
            if ( dir != null )
            {
                dir.mkdirs();
            }
            ObjectOutputStream os = new ObjectOutputStream( new FileOutputStream( cacheFilename ) );
            os.writeObject( persistentCache );
            os.close();
            persistentCache = null;
            iter = null;
            resourceKey = null;
            cacheFile = null;
            dir = null;
            os = null;
        }
        catch ( Throwable t )
        {
            LOG.error( "Unable to save the cache", t );
        }
    }

    /**
     * 
     * @param lvi
     * @return int Will return a status level, VALID, ERROR, UNKNOWN, WARNING
     */
    public LinkValidationResult getCachedResult( LinkValidationItem lvi )
    {
        Iterator iter = getValidators().iterator();
        LinkValidator lv;
        Object resourceKey;
        while ( iter.hasNext() )
        {
            lv = (LinkValidator) iter.next();
            resourceKey = lv.getResourceKey( lvi );
            if ( resourceKey != null && cache.containsKey( resourceKey ) )
            {
                if ( LOG.isDebugEnabled() )
                    LOG.debug( "The cache returns for [" + resourceKey + "] the result [" + cache.get( resourceKey )
                        + "]." );
                return (LinkValidationResult) cache.get( resourceKey );
            }
        }
        lv = null;
        resourceKey = null;
        return null;
    }

    public void setCachedResult( Object resourceKey, LinkValidationResult lvr )
    {
        cache.put( resourceKey, lvr );
    }
}
