/**
 * 
 */
package org.apache.maven.plugin.linkcheck.validation;

import java.io.IOException;
import java.net.URL;

import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.NTCredentials;
import org.apache.commons.httpclient.StatusLine;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.HeadMethod;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Checks links which are normal URLs
 * @author <a href="mailto:bwalding@apache.org">Ben Walding</a>
 * @author <a href="mailto:aheritier@apache.org">Arnaud Heritier</a>
 * @version $Id: HTTPLinkValidator.java 155277 2005-02-24 23:23:29Z aheritier $
 */
public final class OnlineHTTPLinkValidator
    extends HTTPLinkValidator
{
    /**
     * Log for debug output
     */
    private final static Log LOG = LogFactory.getLog( OnlineHTTPLinkValidator.class );

    /** The maximum number of redirections for a link */
    private final static int MAX_NB_REDIRECT = 10;

    /** Use the get method to test pages. */
    private final static String GET_METHOD = "get";

    /** Use the head method to test pages. */
    private final static String HEAD_METHOD = "head";

    private String proxyHost;

    private int proxyPort;

    private String proxyUser;

    private String proxyPass;

    private String proxyNtlmHost;

    private String proxyNtlmDomain;

    private String method = HEAD_METHOD;

    private transient HttpClient cl;

    public OnlineHTTPLinkValidator( String method, String proxyHost, String proxyPort, String proxyUser,
                                    String proxyPass, String proxyNtlmHost, String proxyNtlmDomain )
    {
        if ( LOG.isDebugEnabled() )
            LOG.debug( "Will use method : [" + method + "]" );
        this.method = method;
        if ( proxyHost == null || proxyHost.trim().equals( "" ) )
        {
            this.proxyHost = null;
        }
        else
        {
            this.proxyHost = proxyHost;
            if ( proxyPort != null )
            {
                try
                {
                    this.proxyPort = Integer.parseInt( proxyPort );
                }
                catch ( NumberFormatException e )
                {
                    LOG.warn( "Invalid number for Proxy Port:" + proxyPort );
                    LOG.warn( "Proxy Port won't be used." );
                    this.proxyPort = -1;
                }
            }
            this.proxyUser = proxyUser;
            this.proxyPass = proxyPass;
            if ( proxyNtlmHost != null && !proxyNtlmHost.trim().equals( "" ) )
            {
                this.proxyNtlmHost = proxyNtlmHost;
                this.proxyNtlmDomain = proxyNtlmDomain;
            }
        }
        initHttpClient();
    }

    /**
     * @see org.apache.maven.plugin.linkcheck.LinkValidator#validateLink(org.apache.maven.plugin.linkcheck.LinkValidationItem)
     */
    public LinkValidationResult validateLink( LinkValidationItem lvi )
    {
        if ( cl == null )
            initHttpClient();
        try
        {
            String link = lvi.getLink();
            HttpMethod hm = null;
            try
            {
                hm = checkLink( cl, link, 0 );
            }
            catch ( Throwable t )
            {
                if ( LOG.isDebugEnabled() )
                {
                    LOG
                        .error( "Received: [" + t + "] for [" + lvi.getLink() + "] in page [" + lvi.getSource() + "]",
                                t );
                }
                else
                {
                    LOG.error( "Received: [" + t + "] for [" + lvi.getLink() + "] in page [" + lvi.getSource() + "]" );
                }
                return new LinkValidationResult( LinkValidationResult.ERROR, false, t.getClass().getName() + " : "
                    + t.getMessage() );
            }
            if ( hm == null )
                return new LinkValidationResult( LinkValidationResult.ERROR, false, "Cannot retreive HTTP Status" );
            if ( hm.getStatusCode() == HttpStatus.SC_OK )
            {
                return new LinkValidationResult( LinkValidationResult.VALID, true, hm.getStatusCode() + " "
                    + hm.getStatusText() );
            }
            else
            {
                // If there's a redirection ... add a warning
                if ( hm.getStatusCode() == HttpStatus.SC_MOVED_PERMANENTLY
                    || hm.getStatusCode() == HttpStatus.SC_MOVED_TEMPORARILY
                    || hm.getStatusCode() == HttpStatus.SC_TEMPORARY_REDIRECT )
                {
                    LOG.warn( "Received: [" + hm.getStatusCode() + "] for [" + lvi.getLink() + "] in page ["
                        + lvi.getSource() + "]" );
                    return new LinkValidationResult( LinkValidationResult.WARNING, true, hm.getStatusCode() + " "
                        + hm.getStatusText() );
                }
                else
                {
                    LOG.error( "Received: [" + hm.getStatusCode() + "] for [" + lvi.getLink() + "] in page ["
                        + lvi.getSource() + "]" );
                    return new LinkValidationResult( LinkValidationResult.ERROR, false, hm.getStatusCode() + " "
                        + hm.getStatusText() );
                }
            }

        }
        catch ( Throwable t )
        {
            if ( LOG.isDebugEnabled() )
                LOG.error( "Received: [" + t + "] for [" + lvi.getLink() + "] in page [" + lvi.getSource() + "]", t );
            else
                LOG.error( "Received: [" + t + "] for [" + lvi.getLink() + "] in page [" + lvi.getSource() + "]" );
            return new LinkValidationResult( LinkValidationResult.ERROR, false, t.getMessage() );
        }
    }

    private void initHttpClient()
    {
        LOG.debug( "A new HttpClient instance is needed ..." );
        // Some web servers doesn't allow the default user-agent sent by httpClient
        System.setProperty( "httpclient.useragent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)" );
        cl = new HttpClient( new MultiThreadedHttpConnectionManager() );
        HostConfiguration hc = new HostConfiguration();
        HttpState state = new HttpState();
        if ( this.proxyHost != null )
        {
            hc.setProxy( this.proxyHost, this.proxyPort );
            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "Proxy Host:" + proxyHost );
                LOG.debug( "Proxy Port:" + proxyPort );
            }
            if ( this.proxyUser != null && this.proxyPass != null )
            {
                if ( LOG.isDebugEnabled() )
                    LOG.debug( "Proxy User:" + proxyUser );
                Credentials credentials;
                if ( this.proxyNtlmHost != null )
                {
                    credentials = new NTCredentials( this.proxyUser, this.proxyPass, this.proxyNtlmHost,
                                                     this.proxyNtlmDomain );
                }
                else
                {
                    credentials = new UsernamePasswordCredentials( this.proxyUser, this.proxyPass );
                }
                state.setProxyCredentials( null, null, credentials );
            }

        }
        else
        {
            LOG.debug( "Not using a proxy" );
        }
        cl.setHostConfiguration( hc );
        cl.setState( state );
        LOG.debug( "New HttpClient instance created." );
    }

    private HttpMethod checkLink( HttpClient cl, String link, int nbRedirect )
        throws HttpException, IOException
    {
        if ( nbRedirect > MAX_NB_REDIRECT )
        {
            throw new HttpException( "Maximum number of redirections (" + MAX_NB_REDIRECT + ") exceeded" );
        }
        HttpMethod hm;
        if ( HEAD_METHOD.equals( method ) )
            hm = new HeadMethod( link );
        else
            hm = new GetMethod( link );
        try
        {
            // We want to do it manually
            hm.setFollowRedirects( false );
            URL url = new URL( link );
            cl.getHostConfiguration().setHost( url.getHost(), url.getPort(), url.getProtocol() );
            cl.executeMethod( hm );
            StatusLine sl = hm.getStatusLine();

            if ( sl == null )
            {
                LOG.error( "Unknown error validating link : " + link );
                return null;
            }

            if ( hm.getStatusCode() == HttpStatus.SC_MOVED_PERMANENTLY
                || hm.getStatusCode() == HttpStatus.SC_MOVED_TEMPORARILY
                || hm.getStatusCode() == HttpStatus.SC_TEMPORARY_REDIRECT )
            {
                Header locationHeader = hm.getResponseHeader( "location" );
                if ( locationHeader == null )
                {
                    LOG.error( "Site sent redirect, but did not set Location header" );
                    return hm;
                }
                else
                {
                    String newLink = locationHeader.getValue();
                    // Be careful to absolute/relative links
                    if ( !newLink.startsWith( "http://" ) && !newLink.startsWith( "https://" ) )
                    {
                        if ( newLink.startsWith( "/" ) )
                        {
                            URL oldUrl = new URL( link );
                            newLink = oldUrl.getProtocol() + "://" + oldUrl.getHost()
                                + ( oldUrl.getPort() > 0 ? ":" + oldUrl.getPort() : "" ) + newLink;
                        }
                        else
                        {
                            newLink = link + newLink;
                        }
                    }
                    HttpMethod oldHm = hm;
                    LOG.info( "[" + link + "] is redirected to [" + newLink + "]" );
                    oldHm.releaseConnection();
                    hm = checkLink( cl, newLink, nbRedirect + 1 );
                    // Restore the hm to "Moved permanently" | "Moved temporarily" | "Temporary redirect" 
                    // if the new location is found to allow us to report it
                    if ( hm.getStatusCode() == HttpStatus.SC_OK && nbRedirect == 0)
                    {
                        return oldHm;
                    }
                }
            }

        }
        finally
        {
            hm.releaseConnection();
        }
        return hm;
    }

}
